@@ -20,6 +20,7 @@ try:
     from celery import Task, shared_task, current_app
     from celery.result import AsyncResult
     from celery.exceptions import CeleryError, TimeoutError
+    from celery.utils.time import maybe_iso8601
     from kombu.utils import uuid as task_uuid
     from kombu import serialization
 except ImportError:
@@ -429,7 +430,9 @@ class DjangoTask(Task):
 
     def _trigger(self, result, args, kwargs, invocation_id, task_id=None, eta=None, countdown=None, expires=None,
                  time_limit=None, soft_time_limit=None, stale_time_limit=None, ignore_task_after_success=None,
-                 is_async=True, **options):
+                 is_async=True, headers=None, **options):
+
+        headers = {} if headers is None else headers
 
         app = self._get_app()
 
@@ -455,6 +458,12 @@ class DjangoTask(Task):
             stale_time_limit=stale_time_limit
         ))
 
+        headers.update(dict(
+            apply_time=options['apply_time'].isoformat(),
+            trigger_time=trigger_time.isoformat(),
+            stale_time_limit=stale_time_limit
+        ))
+
         ignore_task_after_success = (
             ignore_task_after_success if ignore_task_after_success is not None
             else self.ignore_task_after_success_timedelta is not None
@@ -479,7 +488,7 @@ class DjangoTask(Task):
             else:
                 result.set_options(options)
                 self.on_invocation_trigger(invocation_id, args, kwargs, task_id, options, result)
-                result.set_result(self._apply_and_get_result(args, kwargs, **options))
+                result.set_result(self._apply_and_get_result(args, kwargs, headers=headers, **options))
 
     def _first_apply(self, args=None, kwargs=None, invocation_id=None, is_async=True, is_on_commit=False, using=None,
                      **options):
@@ -567,6 +576,11 @@ class DjangoTask(Task):
             # In the opposite way task will be failed
             self.on_task_retry(self.request.id, args, kwargs, exc, eta)
 
+        # Fix bug in celery==5.2.*
+        request_execution_options = self.request.as_execution_options()
+        if 'expires' in request_execution_options:
+            options.setdefault('expires', maybe_iso8601(request_execution_options['expires']))
+
         return super().retry(
             args=args, kwargs=kwargs, exc=exc, throw=throw,
             eta=eta, max_retries=max_retries, **options
